<!--

==IIS Log Parser Browser 1.0==

This tool works with Microsoft's Log Parser 2.2 to allow you to run pretty advanced reports on your IIS 
log files. It is a simple ASP script, no install required, no scheduled jobs, and no configuration of the 
script itself - just set the appropriate permissions in IIS.

===INSTALLATION INSTRUCTIONS===
* Install Log Parser 2.2, with COM support, see [www.logparser.com] for more info and download link
* Copy this script to somewhere you can access it through IIS, without attracting too many curious souls.
* Register the "ASPDNS.dll" File with RegSvr32 in a location of your choice OR Set the "EnableReverseIPViewing" variable to "False"
* Set the permissions on the file (or containing folder) to not allow anonymous, and to use integrated 
windows auth. Then when viewing the page use an appropriate administrator group user.


===WARNING / DISCLAIMER===
* The Detail Viewing opens the way to "SQL Injection" attacks, because the WHERE clause is basically querystring-provided. This would be extremely dangerous in a SQL Server, but as far as I know does not matter with Log Parser. In any case, the script will only work (get past the ADSI query) if the client is authenticated as an Administrator. However, if you have any doubts at all, turn off the detail viewing below and you should be set.
* This tool is only intended for casual viewing / exploration of your existing log files:
** The reports are not formatted for public consumption, they are pretty ugly (although that would be easy to fix)
** The reports are very expensive to run, from a server resources point of view - there is no daily "compilation" of files into some internal binary format like most other log analysis tools (eg urchin, analog)
* This script requires you to use some form of integrated IIS authentication so that it is executed with Administrator priviledges. I am no IIS security expert, but my assumption is that you should NEVER do that over a public network (internet) without the communication being protected by SSL (HTTPS). 
* The ASPDNS.dll COM component is created by Stefan Schoeman, and it is considered "DonationWare", so feel free to donate at his page:
** [http://www.internext.co.za/stefan/aspdns/]
* This script does a bunch of complicated things using complicated technologies, there is no way I can provide any guarantees or implied promises of performance, safety, security etc. Every IIS Admin is on his own here. I have done my best to make this tool as safe and foolproof as possible, but I cannot safely say that it will not turn your server into a slag pile, or allow the hacker rats of mars to turn your IIS server into a brain-eating zombie.

===KEY FEATURES===
* Very simple installation, no security concerns except those noted above - script will only work if you are connected with admin credentials, otherwise it will refuse to run (if anonymous) or crash (if connected user does not have sufficient rights).
* No server modifications at all - no writes to disk, no caching, no compilation - just explore the existing log files in interesting ways.
* Reports can be run remotely, as long as secure integrated authentication is possible - if you're analysing IIS logs you probably have web access to this server!
* Various IIS Log stats views, depending on the information that is being logged to the IIS Log Files - the check is per-file, so when you increase logging columns you immediately increase the reporting options available.
* Placeholders for easy access to a specific file (eg yesterday's) with a generic URL.
* Ability to query specific (arbitrary) time slices, with default slicing by hour - ALL TIMES UTC!
* Ability to get detail/raw results for any "by HitCount" result set, and copy / paste results into Excel or other resource for further analysis
* Ability to look up the Reverse DNS records of client IP addresses, only for the reported results - Log Parser itself tries to get the reverse DNS value of EVERY IP address in the WHERE result set, regardless of 
whether it makes the top 50 or not, so this takes a VERY long time - here the reverse DNS lookup is performed only for the results displayed. PLEASE NOTE: If you intend to look at large "Raw" result sets 
(the script normally limits you to the first 2000 records), the reverse DNS lookups will still be done so it may make sense to tun them off before getting large raw result sets.
* Display of the "SQL" that generated the results on every page. This can always be copy/pasted into a command-line for an immediate refinement / modification to the query, with all the power of log parser.
* Easy / Trivial addition of new reports:
** Add the report name (with a new ID) in the list at the top
** Add the report query to the existing list/block, keeping in mind that certain column names have special functionality HitCount will always be summed up:
*** If "ExtraWhere" and "HitCount" are both in the output, the HitCount will become a link with the ExtraWhere data provided.
*** If "c-ip" is in the output and Reverse DNS lookups are enabled, these will be automaticall carried out
** Add the display of the new report in the "Capabilities" area, being sure to check for the required columns.
* Tiny! The script itself is less than 50KB including instructions, no compilation is required, and the DNS lookup DLL is optional.

===POSSIBLE FUTURE EXTENSIONS===
* Work on other input formats, eg XSP, Cassini, Apache, etc
* Update these initial comments to be HTML comments, and so avoid the leading 
* Use client IP stats to create a user density map of the world, like urchin
* Use user-agent information to group into specific browsers, removing "chaff" in the IE stats particularly, like analog
* Add graphical representations for some results, like analog
* Add "missing" record information in all TOP XXX queries, as an "Others" summary entry at the bottom, like analog
* Provide a "Convert to Local Time" option... Complicated because of date boundaries not corresponding to file boundaries anymore.
* Allow for side-by-side comparison of 2 files
* Consider using the Log Parser 2.2 "<1>" website log file syntax, determine how groups of records would be selected (by day?), how this would affect performance on large datasets (does it try to parse all files or only the necessary ones?), and how this would affect the ability to detect logging "Capabilities".
* Consider implementing the same concept for PerfLog CSV files, and possibly BLG files with relog conversion tool.
** Would need to concentrate on "Core" performance counters across all Windows systems + IIS & SQL Server, and provide counter setup files and instructions...


===COPYRIGHT ETC===
* This script Copyright: (C) Tao Klerks 2007 
** Permission to re-distribute this script as long as copyright notice and comments remain intact
* Log Parser 2.2 (C) 2004 Microsoft Corporation. All rights reserved.
* aspdns.dll Copyright: (C) Stefan Schoeman 1999

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


===VERSION HISTORY===
* 1.0 2007-06-10 - Basic features, placed online


===FEEDBACK===
Please contact me (Tao Klerks) with any questions, comments, suggestions, concerns, etc at [www.architectshack.com]

-->

<%

'GLOBAL INSTALL / SECURITY SETTINGS
EnableDetailViewing = False 'Possible security risk with "SQL Injection" when True
EnableReverseIPViewing = False 'Requires additional DNS lookup DLL/COM object and also slows things down a bit (but v useful!)


'Get request details
SiteNumber = Request.QueryString("SiteNumber")
FileName = Request.QueryString("FileName")
TimeFrom = Request.QueryString("TimeFrom")
TimeTo = Request.QueryString("TimeTo")
DetailType = Request.QueryString("DetailType")

If EnableDetailViewing Then ExtraWhere = Request.QueryString("ExtraWhere")

FileDate = ""

'Constants:
DetailType1 = "Hits By hour slice"
DetailType2 = "Top 100 most-hit URI Stem & Status combinations"
DetailType3 = "Top 100 most-hit non-200 non-304 status URI Stems"
DetailType4 = "Top 100 client IPs"
DetailType5 = "Top 100 client IPs connecting on port 80 (unsafe?)"
DetailType6 = "Top 100 URI Stems accessed from port 80 (unsafe?)"
DetailType7 = "All request statuses by Hit Count"
DetailType8 = "Top 100 Longest-running requests"
DetailType9 = "Top 100 Longest-running URI Stems (by max run time)"
DetailType10 = "Top 100 Longest-running URI Stems (by avg run time)"
DetailType11 = "Top 100 Least visited ASP/ASPX Pages (but visited at least once)"
DetailType12 = "Top 100 Data-returning URI Stems (by max bytes sent)"
DetailType13 = "Top 100 Data-returning URI Stems (by avg bytes sent)"
DetailType14 = "Top 100 Data-accepting URI Stems (by avg bytes received)"
DetailType15 = "Top 100 User-Agents"
DetailType16 = "Top 100 Broken Link (404) Referrers and Referenced URI Stems"
DetailType17 = "All extensions by hit count"
DetailType18 = "All extensions by total bytes sent"


Call OpenXHTMLDocument("IIS Log Parser / Browser 1.0")

'First Check whether we should be bothering to try to run. If there is no logon user then we don't 
'have the credentials, so no point in continuing this script (unless something is drastically 
'wrong with the site's permissions, in which case the administrator needs to figure this out, 
'and possibly edit this scrit to bypass this check.

If Request.ServerVariables("LOGON_USER") <> "" Then
	'We have to assume that everything is ok
Else
	Response.Write "This script must be run from a priviledged account - please disable anonymous access to this script."
	Response.End
End If

'Site validation (security)
If SiteNumber <> "" Then 
	If IsNumeric(SiteNumber) Then 
		SiteNumber = CLng(SiteNumber) 
	Else 
		SiteNumber = ""
	End If
End If

SiteServer = Request.ServerVariables("SERVER_NAME")
SiteServerIP = Request.ServerVariables("LOCAL_ADDR")
Response.Write "<h3><a href=""?"">" & SiteServer & " (" & SiteServerIP & ")</a></h3>"

If SiteNumber <> "" Then
	Set objChildNode = GetObject("IIS://LOCALHOST/W3SVC/" & SiteNumber)
	Call DisplaySiteInfo (objChildNode, SiteLogPath)

	'Filename Validation
	If FileName <> "" Then
		'First Replace Placeholders
		If FileName = "<TODAY>" Then FileName = GetLogFileName(Now)
		If FileName = "<YESTERDAY>" Then FileName = GetLogFileName(DateAdd("d", -1, Now))
		If FileName = "<AWEEKAGO>" Then FileName = GetLogFileName(DateAdd("d", -7, Now))
	
		Set objChildNode = GetObject("IIS://LOCALHOST/W3SVC/" & SiteNumber)
	
		'Then use FSO to confirm existence
		Set fso = CreateObject("Scripting.FileSystemObject")
		Set FolderObject = fso.GetFolder(SiteLogPath)
		
		For Each File in FolderObject.Files
			If WildCardMatch (File.Name, FileName) Then 
				FileName = File.Name
				FileDate = FormatDateTime(File.DateLastModified, 2)
			End If
		Next
		
		If Instr(FileName, "*") Then 
			FileName = ""
			TimeFrom = ""
			TimeTo = ""
		Else
			'Validate Times
			If TimeFrom <> "" Then
				If IsDate(FileDate & " " & TimeFrom) Then
					'everything's fine I guess...
				Else
					TimeFrom = ""
				End If
			End If
			
			If TimeTo <> "" Then
				If IsDate(FileDate & " " & TimeTo) Then
					'everything's fine I guess...
				Else
					TimeTo = ""
				End If
			End If
		End If

	Else
		TimeFrom = ""
		TimeTo = ""
	End If


	If FileName <> "" Then

		Set fso = CreateObject("Scripting.FileSystemObject")
		Set FileObject = fso.GetFile(SiteLogPath & "\" & FileName)

   	Response.Write "<h5>"
		Call DisplayFileLink (SiteNumber, FileObject)
   	Response.Write "</h5>" & vbCrLf
		
		If TimeFrom <> "" And TimeTo <> "" Then

			'Display the time Link
	   	Response.Write "<h5>"
			Call DisplayTimeLink (SiteNumber, FileObject, TimeFrom, TimeTo)
	   	Response.Write "</h5>" & vbCrLf

		End If
		
		If DetailType <> "" Then

			'Display the selected Option
	   	Response.Write "<h5>"
			Call DisplayDetailLink (SiteNumber, FileObject, TimeFrom, TimeTo, DetailType)
	   	Response.Write "</h5>" & vbCrLf

			'Create / display the detail for the selected option
			If ExtraWhere <> "" Then
				strQuery = "SELECT TOP 2000 * " 
				strQuery = strQuery & " FROM " & Replace(SiteLogPath & "\" & FileName, """", """""") 
				strQuery = strQuery & " WHERE 1 = 1 "
				If TimeFrom <> "" And TimeTo <> "" Then 
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') >= '" & TimeFrom & "' " 
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') <= '" & TimeTo & "'"
				End If
				strQuery = strQuery & ExtraWhere
			ElseIf DetailType = DetailType1 Then
				strQuery = "SELECT TO_STRING(time, 'HH') AS Hour, count(*) As HitCount "
				strQuery = strQuery & " FROM " & Replace(SiteLogPath & "\" & FileName, """", """""") 
				strQuery = strQuery & " WHERE 1 = 1 "
				strQuery = strQuery & " GROUP BY Hour ORDER BY Hour ASC"
			ElseIf DetailType = DetailType2 Then
				strQuery = "SELECT TOP 100 sc-status, cs-uri-stem, count(sc-status) As HitCount "
				If EnableDetailViewing Then strQuery = strQuery & "   , STRCAT(' AND cs-uri-stem = \'', STRCAT(REPLACE_STR(cs-uri-stem, '\'', '\\\''), STRCAT('\' AND sc-status = ', TO_STRING(sc-status)))) AS ExtraWhere " 
				strQuery = strQuery & " FROM " & Replace(SiteLogPath & "\" & FileName, """", """""") 
				strQuery = strQuery & " WHERE 1 = 1 "
				If TimeFrom <> "" And TimeTo <> "" Then 
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') >= '" & TimeFrom & "' " 
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') <= '" & TimeTo & "' "
				End If
				strQuery = strQuery & " GROUP BY sc-status, cs-uri-stem ORDER BY HitCount DESC"
			ElseIf DetailType = DetailType3 Then
				strQuery = "SELECT TOP 100 sc-status, cs-uri-stem, count(sc-status) As HitCount "
				If EnableDetailViewing Then strQuery = strQuery & "   , STRCAT(' AND sc-status <> 200 AND sc-status <> 304 AND cs-uri-stem = \'', STRCAT(REPLACE_STR(cs-uri-stem, '\'', '\\\''), STRCAT('\' AND sc-status = ', TO_STRING(sc-status)))) AS ExtraWhere "
				strQuery = strQuery & " FROM " & Replace(SiteLogPath & "\" & FileName, """", """""") 
				strQuery = strQuery & " WHERE sc-status <> 200 AND sc-status <> 304 "
				If TimeFrom <> "" And TimeTo <> "" Then 
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') >= '" & TimeFrom & "' "
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') <= '" & TimeTo & "' "
				End If
				strQuery = strQuery & " GROUP BY sc-status, cs-uri-stem ORDER BY HitCount DESC"
			ElseIf DetailType = DetailType4 Then
				strQuery = "SELECT TOP 100 c-ip, count(c-ip) As HitCount " 
				If EnableDetailViewing Then strQuery = strQuery & "   , STRCAT(' AND c-ip = \'', STRCAT(REPLACE_STR(c-ip, '\'', '\\\''), '\'')) AS ExtraWhere " 
				strQuery = strQuery & " FROM " & Replace(SiteLogPath & "\" & FileName, """", """""") 
				strQuery = strQuery & " WHERE 1 = 1 "
				If TimeFrom <> "" And TimeTo <> "" Then 
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') >= '" & TimeFrom & "' " 
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') <= '" & TimeTo & "' "
				End If
				strQuery = strQuery & " GROUP BY c-ip ORDER BY HitCount DESC"
			ElseIf DetailType = DetailType5 Then
				strQuery = "SELECT TOP 100 c-ip, count(c-ip) As HitCount " 
				If EnableDetailViewing Then strQuery = strQuery & "   , STRCAT(' AND s-port = 80 AND c-ip = \'', STRCAT(REPLACE_STR(c-ip, '\'', '\\\''), '\'')) AS ExtraWhere " 
				strQuery = strQuery & " FROM " & Replace(SiteLogPath & "\" & FileName, """", """""") 
				strQuery = strQuery & " WHERE s-port = 80 " 
				If TimeFrom <> "" And TimeTo <> "" Then 
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') >= '" & TimeFrom & "' "
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') <= '" & TimeTo & "' "
				End If
				strQuery = strQuery & " GROUP BY c-ip ORDER BY HitCount DESC"
			ElseIf DetailType = DetailType6 Then
				strQuery = "SELECT TOP 100 cs-uri-stem, count(cs-uri-stem) As HitCount "
				If EnableDetailViewing Then strQuery = strQuery & "   , STRCAT(' AND s-port = 80 AND cs-uri-stem = \'', STRCAT(REPLACE_STR(cs-uri-stem, '\'', '\\\''), '\'')) AS ExtraWhere "
				strQuery = strQuery & " FROM " & Replace(SiteLogPath & "\" & FileName, """", """""") 
				strQuery = strQuery & " WHERE s-port = 80 "
				If TimeFrom <> "" And TimeTo <> "" Then 
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') >= '" & TimeFrom & "' "
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') <= '" & TimeTo & "' "
				End If
				strQuery = strQuery & " GROUP BY cs-uri-stem ORDER BY HitCount DESC"
			ElseIf DetailType = DetailType7 Then
				strQuery = "SELECT sc-status, count(sc-status) As HitCount "
				If EnableDetailViewing Then strQuery = strQuery & "   , STRCAT(' AND sc-status = ', TO_STRING(sc-status)) AS ExtraWhere "
				strQuery = strQuery & " FROM " & Replace(SiteLogPath & "\" & FileName, """", """""") 
				strQuery = strQuery & " WHERE 1 = 1 "
				If TimeFrom <> "" And TimeTo <> "" Then 
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') >= '" & TimeFrom & "' "
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') <= '" & TimeTo & "' "
				End If
				strQuery = strQuery & " GROUP BY sc-status ORDER BY HitCount DESC"
			ElseIf DetailType = DetailType8 Then
				strQuery = "SELECT TOP 100 * "
				strQuery = strQuery & " FROM " & Replace(SiteLogPath & "\" & FileName, """", """""") 
				strQuery = strQuery & " WHERE 1 = 1 "
				If TimeFrom <> "" And TimeTo <> "" Then 
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') >= '" & TimeFrom & "' "
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') <= '" & TimeTo & "' "
				End If
				strQuery = strQuery & " ORDER BY time-taken DESC"
			ElseIf DetailType = DetailType9 Then
				strQuery = "SELECT TOP 100 cs-uri-stem, count(cs-uri-stem) As HitCount, Min(time-taken) As MinTime, Avg(time-taken) As AvgTime, Max(time-taken) As MaxTime, Sum(time-taken) As TotalTime "
				If EnableDetailViewing Then strQuery = strQuery & "   , STRCAT(' AND cs-uri-stem = \'', STRCAT(REPLACE_STR(cs-uri-stem, '\'', '\\\''), '\'')) AS ExtraWhere " 
				strQuery = strQuery & " FROM " & Replace(SiteLogPath & "\" & FileName, """", """""") 
				strQuery = strQuery & " WHERE 1 = 1 "
				If TimeFrom <> "" And TimeTo <> "" Then 
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') >= '" & TimeFrom & "' "
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') <= '" & TimeTo & "' "
				End If
				strQuery = strQuery & " GROUP BY cs-uri-stem ORDER BY MaxTime DESC"
			ElseIf DetailType = DetailType10 Then
				strQuery = "SELECT TOP 100 cs-uri-stem, count(cs-uri-stem) As HitCount, Min(time-taken) As MinTime, Avg(time-taken) As AvgTime, Max(time-taken) As MaxTime, Sum(time-taken) As TotalTime "
				If EnableDetailViewing Then strQuery = strQuery & "   , STRCAT(' AND cs-uri-stem = \'', STRCAT(REPLACE_STR(cs-uri-stem, '\'', '\\\''), '\'')) AS ExtraWhere " 
				strQuery = strQuery & " FROM " & Replace(SiteLogPath & "\" & FileName, """", """""") 
				strQuery = strQuery & " WHERE 1 = 1 "
				If TimeFrom <> "" And TimeTo <> "" Then 
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') >= '" & TimeFrom & "' "
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') <= '" & TimeTo & "' "
				End If
				strQuery = strQuery & " GROUP BY cs-uri-stem ORDER BY AvgTime DESC"
			ElseIf DetailType = DetailType11 Then
				strQuery = "SELECT TOP 100 cs-uri-stem, count(cs-uri-stem) As HitCount "
				If EnableDetailViewing Then strQuery = strQuery & "   , STRCAT(' AND cs-uri-stem = \'', STRCAT(REPLACE_STR(cs-uri-stem, '\'', '\\\''), '\'')) AS ExtraWhere " 
				strQuery = strQuery & " FROM " & Replace(SiteLogPath & "\" & FileName, """", """""") 
				strQuery = strQuery & " WHERE (cs-uri-stem LIKE '%.asp' OR cs-uri-stem LIKE '%.aspx') "
				If TimeFrom <> "" And TimeTo <> "" Then 
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') >= '" & TimeFrom & "' "
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') <= '" & TimeTo & "' "
				End If
				strQuery = strQuery & " GROUP BY cs-uri-stem ORDER BY HitCount ASC"
			ElseIf DetailType = DetailType12 Then
				strQuery = "SELECT TOP 100 cs-uri-stem, count(cs-uri-stem) As HitCount, Min(sc-bytes) As MinSent, Avg(sc-bytes) As AvgSent, Max(sc-bytes) As MaxSent, Sum(sc-bytes) As TotalSent "
				If EnableDetailViewing Then strQuery = strQuery & "   , STRCAT(' AND cs-uri-stem = \'', STRCAT(REPLACE_STR(cs-uri-stem, '\'', '\\\''), '\'')) AS ExtraWhere " 
				strQuery = strQuery & " FROM " & Replace(SiteLogPath & "\" & FileName, """", """""") 
				strQuery = strQuery & " WHERE 1 = 1 "
				If TimeFrom <> "" And TimeTo <> "" Then 
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') >= '" & TimeFrom & "' "
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') <= '" & TimeTo & "' "
				End If
				strQuery = strQuery & " GROUP BY cs-uri-stem ORDER BY MaxSent DESC"
			ElseIf DetailType = DetailType13 Then
				strQuery = "SELECT TOP 100 cs-uri-stem, count(cs-uri-stem) As HitCount, Min(sc-bytes) As MinSent, Avg(sc-bytes) As AvgSent, Max(sc-bytes) As MaxSent, Sum(sc-bytes) As TotalSent "
				If EnableDetailViewing Then strQuery = strQuery & "   , STRCAT(' AND cs-uri-stem = \'', STRCAT(REPLACE_STR(cs-uri-stem, '\'', '\\\''), '\'')) AS ExtraWhere " 
				strQuery = strQuery & " FROM " & Replace(SiteLogPath & "\" & FileName, """", """""") 
				strQuery = strQuery & " WHERE 1 = 1 "
				If TimeFrom <> "" And TimeTo <> "" Then 
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') >= '" & TimeFrom & "' "
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') <= '" & TimeTo & "' "
				End If
				strQuery = strQuery & " GROUP BY cs-uri-stem ORDER BY AvgSent DESC"
			ElseIf DetailType = DetailType14 Then
				strQuery = "SELECT TOP 100 cs-uri-stem, count(cs-uri-stem) As HitCount, Min(cs-bytes) As MinReceived, Avg(cs-bytes) As AvgReceived, Max(cs-bytes) As MaxReceived, Sum(cs-bytes) As TotalReceived "
				If EnableDetailViewing Then strQuery = strQuery & "   , STRCAT(' AND cs-uri-stem = \'', STRCAT(REPLACE_STR(cs-uri-stem, '\'', '\\\''), '\'')) AS ExtraWhere " 
				strQuery = strQuery & " FROM " & Replace(SiteLogPath & "\" & FileName, """", """""") 
				strQuery = strQuery & " WHERE 1 = 1 "
				If TimeFrom <> "" And TimeTo <> "" Then 
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') >= '" & TimeFrom & "' "
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') <= '" & TimeTo & "' "
				End If
				strQuery = strQuery & " GROUP BY cs-uri-stem ORDER BY AvgReceived DESC"
			ElseIf DetailType = DetailType15 Then
				strQuery = "SELECT TOP 100 cs(User-Agent), count(*) As HitCount "
				If EnableDetailViewing Then strQuery = strQuery & "   , STRCAT(' AND cs(User-Agent) = \'', STRCAT(REPLACE_STR(cs(User-Agent), '\'', '\\\''), '\'')) AS ExtraWhere " 
				strQuery = strQuery & " FROM " & Replace(SiteLogPath & "\" & FileName, """", """""") 
				strQuery = strQuery & " WHERE 1 = 1 "
				If TimeFrom <> "" And TimeTo <> "" Then 
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') >= '" & TimeFrom & "' "
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') <= '" & TimeTo & "' "
				End If
				strQuery = strQuery & " GROUP BY cs(User-Agent) ORDER BY HitCount DESC"
			ElseIf DetailType = DetailType16 Then
				strQuery = "SELECT TOP 100 cs(Referer), cs-uri-stem, count(*) As HitCount "
				If EnableDetailViewing Then strQuery = strQuery & "   , STRCAT(' AND sc-status = 404 AND cs(Referer) = \'', STRCAT(REPLACE_STR(cs(Referer), '\'', '\\\''), STRCAT('\' AND cs-uri-stem = \'', STRCAT(REPLACE_STR(cs-uri-stem, '\'', '\\\''), '\'')))) AS ExtraWhere " 
				strQuery = strQuery & " FROM " & Replace(SiteLogPath & "\" & FileName, """", """""") 
				strQuery = strQuery & " WHERE sc-status = 404 "
				If TimeFrom <> "" And TimeTo <> "" Then 
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') >= '" & TimeFrom & "' "
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') <= '" & TimeTo & "' "
				End If
				strQuery = strQuery & " GROUP BY cs(Referer), cs-uri-stem ORDER BY HitCount DESC"
			ElseIf DetailType = DetailType17 Then
				strQuery = "SELECT TOP 100 EXTRACT_EXTENSION(cs-uri-stem) As URIExtension, count(*) As HitCount "
				If EnableDetailViewing Then strQuery = strQuery & "   , STRCAT(' AND EXTRACT_EXTENSION(cs-uri-stem) = \'', STRCAT(REPLACE_STR(EXTRACT_EXTENSION(cs-uri-stem), '\'', '\\\''), '\'')) AS ExtraWhere " 
				strQuery = strQuery & " FROM " & Replace(SiteLogPath & "\" & FileName, """", """""") 
				strQuery = strQuery & " WHERE 1 = 1 "
				If TimeFrom <> "" And TimeTo <> "" Then 
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') >= '" & TimeFrom & "' "
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') <= '" & TimeTo & "' "
				End If
				strQuery = strQuery & " GROUP BY URIExtension ORDER BY HitCount DESC"
			ElseIf DetailType = DetailType18 Then
				strQuery = "SELECT TOP 100 EXTRACT_EXTENSION(cs-uri-stem) As URIExtension, count(*) As HitCount, Min(sc-bytes) As MinSent, Avg(sc-bytes) As AvgSent, Max(sc-bytes) As MaxSent, Sum(sc-bytes) As TotalSent "
				If EnableDetailViewing Then strQuery = strQuery & "   , STRCAT(' AND EXTRACT_EXTENSION(cs-uri-stem) = \'', STRCAT(REPLACE_STR(EXTRACT_EXTENSION(cs-uri-stem), '\'', '\\\''), '\'')) AS ExtraWhere " 
				strQuery = strQuery & " FROM " & Replace(SiteLogPath & "\" & FileName, """", """""") 
				strQuery = strQuery & " WHERE 1 = 1 "
				If TimeFrom <> "" And TimeTo <> "" Then 
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') >= '" & TimeFrom & "' "
					strQuery = strQuery & " AND TO_STRING(time, 'hh:mm') <= '" & TimeTo & "' "
				End If
				strQuery = strQuery & " GROUP BY URIExtension ORDER BY TotalSent DESC"
			ElseIf DetailType = "" Then
				Response.Write "No Detail Type!"
			Else
				Response.Write "Invalid Detail Type!"
			End If

			Response.Write " <b>" & strQuery & "</b> <br /> <br /> " & VBCrLf
			Response.Flush
			
			If strQuery <> "" Then
				Set oLogQuery = CreateObject("MSUtil.LogQuery")
				Set oRecordSet = oLogQuery.Execute(strQuery)
				
				ColumnCount = oRecordSet.getColumnCount()

				TotalColumnNumber = -1
				HourColumnNumber = -1
				ExtraWhereColumnNumber = -1
				ClientIPColumnNumber = -1

				TotalHitCountCount = 0
				i = 0
		
				Response.Write "<table>"
				
				Response.Write "<tr>"
				While i < ColumnCount

					If oRecordSet.getColumnName(i) = "ExtraWhere" Then
						ExtraWhereColumnNumber = i
					Else
						Response.Write "<td><b>"
						Response.Write oRecordSet.getColumnName(i)
						Response.Write "</b></td>"
						
						If oRecordSet.getColumnName(i) = "HitCount" Then TotalColumnNumber = i
						If oRecordSet.getColumnName(i) = "Hour" Then HourColumnNumber = i
						If oRecordSet.getColumnName(i) = "c-ip" Then ClientIPColumnNumber = i
					End If

					i = i + 1
				Wend
				Response.Write "</tr>" & vbCrLf
				
				If ClientIPColumnNumber >= 0 And EnableReverseIPViewing Then
					Set DNS = CreateObject("ASPDNS.DNSLookup")
				End If
		
				Do While Not oRecordSet.atEnd
					i = 0
					Set oRecord = oRecordSet.getRecord
		
					Response.Write "<tr>"
					While i < ColumnCount
					
						If HourColumnNumber = i Then
							Response.Write "<td>"
							Call DisplayTimeLink (SiteNumber, FileObject, oRecord.getValue(i) & ":00", oRecord.getValue(i) & ":59")
							Response.Write "</td>"
						ElseIf ExtraWhereColumnNumber = i Then
							'Do Nothing
						ElseIf TotalColumnNumber = i Then 
							TotalHitCountCount = TotalHitCountCount + oRecord.getValue(i)
							If ExtraWhereColumnNumber > -1 And EnableDetailViewing Then
								'Display with link
								Response.Write "<td>"
								Call DisplayDetailExtraWhereLink (SiteNumber, FileObject, TimeFrom, TimeTo, DetailType, oRecord.getValue(i), oRecord.getValue(ExtraWhereColumnNumber))
								Response.Write "</td>"
							Else
								'Display normal
								Response.Write "<td>" & oRecord.getValue(i) & "</td>"
							End If
						ElseIf ClientIPColumnNumber = i Then 
							If EnableReverseIPViewing Then
								Response.Write "<td>" & DNS.GetNameFromIP(oRecord.getValue(i)) & " (" & oRecord.getValue(i) & ")</td>"
							Else
								'Display normal
								Response.Write "<td>" & oRecord.getValue(i) & "</td>"
							End If
						Else
							Response.Write "<td>" & oRecord.getValue(i) & "</td>"
						End If

						i = i + 1
					Wend
					Response.Write "</tr>" & vbCrLf
					
					oRecordSet.MoveNext
				Loop
				oRecordSet.Close

				If TotalHitCountCount > 0 Then
					i = 0

					Response.Write "<tr>"
					While i < ColumnCount
						If TotalColumnNumber = i Then 
							Response.Write "<td><b>" & TotalHitCountCount & "</b></td>"
						Else
							Response.Write "<td>&nbsp;</td>"
						End If
							
						i = i + 1
					Wend
					Response.Write "</tr>" & vbCrLf
				End If
		
				Response.Write "</table>"
		
			End If
			
		Else
		
			Set oLogQuery = CreateObject("MSUtil.LogQuery")
			Set oRecordSet = oLogQuery.Execute("SELECT TOP 1 * FROM " & Replace(SiteLogPath & "\" & FileName, """", """"""))

			ColumnCount = oRecordSet.getColumnCount()
			AvailableColumns = "|"
			
			Do While Not oRecordSet.atEnd
				i = 0
				Set oRecord = oRecordSet.getRecord
				Response.Write "<ul>"
				While i < ColumnCount
					ThisColumnName = oRecordSet.getColumnName(i)

					'Confirm the value is not null - Log Parser uses its own full set of columns even when IIS is not logging much.
					'   however user-agent and referer are often genuinely null... so exceptions for them.
					If Not(IsNull(oRecord.getValue(i))) Or ThisColumnName = "cs(User-Agent)" Or ThisColumnName = "cs(Referer)" Then 
						Response.Write "<li>" & ThisColumnName & "</li>"
						AvailableColumns = AvailableColumns & ThisColumnName & "|"
					End If
					
					i = i + 1
				Wend
				Response.Write "</ul>" & vbCrLf
				oRecordSet.MoveNext
			Loop
			oRecordSet.Close
			
			Response.Write "<ul>"

			If TimeFrom <> "" And TimeTo <> "" Then
				'Don't display the time slice breakout report - we already have time data!
			ElseIf InStr(AvailableColumns, "|time|") Then
				'Do display the time slice breakout report, e do not have time data and it is an available column!
				Response.Write "<li>"
				Call DisplayDetailLink (SiteNumber, FileObject, TimeFrom, TimeTo, DetailType1)
				Response.Write "</li>" & vbCrLf
			End If

			If InStr(AvailableColumns, "|sc-status|") And InStr(AvailableColumns, "|cs-uri-stem|") Then
				Response.Write "<li>"
				Call DisplayDetailLink (SiteNumber, FileObject, TimeFrom, TimeTo, DetailType2)
				Response.Write "</li>" & vbCrLf

				Response.Write "<li>"
				Call DisplayDetailLink (SiteNumber, FileObject, TimeFrom, TimeTo, DetailType3)
				Response.Write "</li>" & vbCrLf
			End If

			If InStr(AvailableColumns, "|c-ip|") Then
				Response.Write "<li>"
				Call DisplayDetailLink (SiteNumber, FileObject, TimeFrom, TimeTo, DetailType4)
				Response.Write "</li>" & vbCrLf
			End If

			If InStr(AvailableColumns, "|c-ip|") And InStr(AvailableColumns, "|s-port|") Then
				Response.Write "<li>"
				Call DisplayDetailLink (SiteNumber, FileObject, TimeFrom, TimeTo, DetailType5)
				Response.Write "</li>" & vbCrLf
			End If

			If InStr(AvailableColumns, "|cs-uri-stem|") And InStr(AvailableColumns, "|s-port|") Then
				Response.Write "<li>"
				Call DisplayDetailLink (SiteNumber, FileObject, TimeFrom, TimeTo, DetailType6)
				Response.Write "</li>" & vbCrLf
			End If

			If InStr(AvailableColumns, "|sc-status|") Then
				Response.Write "<li>"
				Call DisplayDetailLink (SiteNumber, FileObject, TimeFrom, TimeTo, DetailType7)
				Response.Write "</li>" & vbCrLf
			End If

			If InStr(AvailableColumns, "|time-taken|") Then
				Response.Write "<li>"
				Call DisplayDetailLink (SiteNumber, FileObject, TimeFrom, TimeTo, DetailType8)
				Response.Write "</li>" & vbCrLf
			End If

			If InStr(AvailableColumns, "|time-taken|") And InStr(AvailableColumns, "|cs-uri-stem|") Then
				Response.Write "<li>"
				Call DisplayDetailLink (SiteNumber, FileObject, TimeFrom, TimeTo, DetailType9)
				Response.Write "</li>" & vbCrLf

				Response.Write "<li>"
				Call DisplayDetailLink (SiteNumber, FileObject, TimeFrom, TimeTo, DetailType10)
				Response.Write "</li>" & vbCrLf
			End If

			If InStr(AvailableColumns, "|cs-uri-stem|") Then
				Response.Write "<li>"
				Call DisplayDetailLink (SiteNumber, FileObject, TimeFrom, TimeTo, DetailType11)
				Response.Write "</li>" & vbCrLf
			End If

			If InStr(AvailableColumns, "|sc-bytes|") And InStr(AvailableColumns, "|cs-uri-stem|") Then
				Response.Write "<li>"
				Call DisplayDetailLink (SiteNumber, FileObject, TimeFrom, TimeTo, DetailType12)
				Response.Write "</li>" & vbCrLf

				Response.Write "<li>"
				Call DisplayDetailLink (SiteNumber, FileObject, TimeFrom, TimeTo, DetailType13)
				Response.Write "</li>" & vbCrLf
			End If

			If InStr(AvailableColumns, "|cs-bytes|") And InStr(AvailableColumns, "|cs-uri-stem|") Then
				Response.Write "<li>"
				Call DisplayDetailLink (SiteNumber, FileObject, TimeFrom, TimeTo, DetailType14)
				Response.Write "</li>" & vbCrLf
			End If

			If InStr(AvailableColumns, "|cs(User-Agent)|") Then
				Response.Write "<li>"
				Call DisplayDetailLink (SiteNumber, FileObject, TimeFrom, TimeTo, DetailType15)
				Response.Write "</li>" & vbCrLf
			End If

			If InStr(AvailableColumns, "|cs(Referer)|") And InStr(AvailableColumns, "|cs-uri-stem|") Then
				Response.Write "<li>"
				Call DisplayDetailLink (SiteNumber, FileObject, TimeFrom, TimeTo, DetailType16)
				Response.Write "</li>" & vbCrLf
			End If

			If InStr(AvailableColumns, "|cs-uri-stem|") Then
				Response.Write "<li>"
				Call DisplayDetailLink (SiteNumber, FileObject, TimeFrom, TimeTo, DetailType17)
				Response.Write "</li>" & vbCrLf
			End If

			If InStr(AvailableColumns, "|cs-uri-stem|") And InStr(AvailableColumns, "|sc-bytes|") Then
				Response.Write "<li>"
				Call DisplayDetailLink (SiteNumber, FileObject, TimeFrom, TimeTo, DetailType18)
				Response.Write "</li>" & vbCrLf
			End If

			Response.Write "</ul>"
			
		End If
	
	Else
		'Display available files, it looks like one was not selected yet.

		Set fso = CreateObject("Scripting.FileSystemObject")
		Set folder = fso.GetFolder(SiteLogPath)
	
		'Display file links for all files in the log folder of the site
		Response.Write "<ul>"
	   For Each file in folder.Files
	   	If fso.GetExtensionName(file.Path) = "log" Then 
		   	Response.Write "<li>"
		   	Call DisplayFileLink (SiteNumber, file)
		   	Response.Write "</li>" & vbCrLf
		   End If
	   Next
		Response.Write "</ul>"

		'Display aditional "Special" links 
		Response.Write "<ul>"
   	Response.Write "<li>"
	   Call DisplaySpecialFileLink (SiteNumber, "<TODAY>")
	   Response.Write "</li>" & vbCrLf
   	Response.Write "<li>"
	   Call DisplaySpecialFileLink (SiteNumber, "<YESTERDAY>")
	   Response.Write "</li>" & vbCrLf
   	Response.Write "<li>"
	   Call DisplaySpecialFileLink (SiteNumber, "<AWEEKAGO>>")
	   Response.Write "</li>" & vbCrLf
		Response.Write "</ul>"

	End If

Else

	Response.Write "<hr />"

	'Display available sites, it looks like one was not selected yet.
	Set objBaseNode = GetObject("IIS://LOCALHOST/W3SVC")
	For Each objChildNode In objBaseNode
		If objChildNode.class = "IIsWebServer" Then
			Call DisplaySiteInfo (objChildNode, SomePath)
			Response.Write "<hr />" & vbCrLf
		End If
	Next
	Set objBaseNode = Nothing
End If 

Call CloseXHTMLDocument()

'All Done!



'Functions, used above.

Function DisplaySiteInfo (WebSiteObject, ByRef SiteLogPath)

	Dim SiteName, SiteNumber, SiteBindingsArray

	SiteName = WebSiteObject.ServerComment
	SiteNumber = WebSiteObject.Name
	SiteStatus = IISStateText(WebSiteObject.ServerState)
	SiteLogPath = WebSiteObject.LogFileDirectory & "\W3SVC" & SiteNumber
	SiteBindingsArray = WebSiteObject.ServerBindings

	Response.Write "<h3><a href=""?SiteNumber=" & Server.URLEncode(SiteNumber) & """>" & Server.HTMLEncode(SiteName) & "</a> (/W3SVC/" & Server.HTMLEncode(SiteNumber) & "): " & SiteStatus & "</h3>" & vbCrLf
	Response.Write "<ul>"
	For Each strBinding In SiteBindingsArray
   	Response.Write "<li>" & Server.HTMLEncode(strBinding) & "</li>" & vbCrLf
   Next
	Response.Write "</ul>"
   Response.Write "<h4>" & Server.HTMLEncode(SiteLogPath) & "</h4>" & vbCrLf

End Function

Function DisplayFileLink (SiteNumber, FileObject)
	Response.Write "<a href=""?SiteNumber=" & Server.URLEncode(SiteNumber) & _
		"&amp;FileName=" & Server.URLEncode(FileObject.Name) & _
		""">" & Server.HTMLEncode(FileObject.Name) & "</a> (" & _
		Server.HTMLEncode(FileSizeText(FileObject.Size)) & ")"
End Function

Function DisplaySpecialFileLink (SiteNumber, FilePlaceholder)
	Response.Write "Special: <a href=""?SiteNumber=" & Server.URLEncode(SiteNumber) & _
		"&amp;FileName=" & Server.URLEncode(FilePlaceholder) & _
		""">" & Server.HTMLEncode(FilePlaceholder) & "</a>"
End Function

Function DisplayTimeLink (SiteNumber, FileObject, TimeFrom, TimeTo)
	Response.Write "<a href=""?SiteNumber=" & Server.URLEncode(SiteNumber) & _
		"&amp;FileName=" & Server.URLEncode(FileObject.Name) & _
		"&amp;TimeFrom=" & Server.URLEncode(TimeFrom) & _
		"&amp;TimeTo=" & Server.URLEncode(TimeTo) & _
		""">" & Server.HTMLEncode(TimeFrom & "-" & TimeTo) & "</a>"
End Function

Function DisplayDetailLink (SiteNumber, FileObject, TimeFrom, TimeTo, DetailType)
	Response.Write "<a href=""?SiteNumber=" & Server.URLEncode(SiteNumber) & _
		"&amp;FileName=" & Server.URLEncode(FileObject.Name) & _
		"&amp;TimeFrom=" & Server.URLEncode(TimeFrom) & _
		"&amp;TimeTo=" & Server.URLEncode(TimeTo) & _
		"&amp;DetailType=" & Server.URLEncode(DetailType) & _
		""">" & Server.HTMLEncode(DetailType) & "</a>"
End Function

Function DisplayDetailExtraWhereLink (SiteNumber, FileObject, TimeFrom, TimeTo, DetailType, HitCount, ExtraWhere)
	Response.Write "<a href=""?SiteNumber=" & Server.URLEncode(SiteNumber) & _
		"&amp;FileName=" & Server.URLEncode(FileObject.Name) & _
		"&amp;TimeFrom=" & Server.URLEncode(TimeFrom) & _
		"&amp;TimeTo=" & Server.URLEncode(TimeTo) & _
		"&amp;DetailType=" & Server.URLEncode(DetailType) & _
		"&amp;ExtraWhere=" & Server.URLEncode(ExtraWhere) & _
		""">" & Server.HTMLEncode(HitCount) & "</a>"
End Function

Function GetLogFileName (TargetDate)
	GetLogFileName = "ex" & Right(Year(TargetDate), 2) & Right("0" & Month(TargetDate), 2) & Right("0" & Day(TargetDate), 2) & "*.log"
End Function

Function WildCardMatch (RawString, WildcardString)
	'Partial implementation only, I am being lazy - assume 1 and only 1 asterisk.

	If RawString = WildcardString Then 
		WildCardMatch = True
	ElseIf Instr(WildcardString, "*") Then
		If Left(RawString, Instr(WildcardString, "*")-1) = Left(WildcardString, Instr(WildcardString, "*")-1) _
			And Right(RawString, Len(WildcardString) - InstrRev(WildcardString, "*")) = Right(WildcardString, Len(WildcardString) - InstrRev(WildcardString, "*")) Then 
			WildCardMatch = True 
		Else
			WildCardMatch = False 
		End If
	Else 
		WildCardMatch = False 
	End If

End Function

Function IISStateText(StateID)
    SELECT CASE StateID
    CASE 1
        IISStateText = "Starting"
    CASE 2
        IISStateText = "Started"
    CASE 3
        IISStateText = "Stopping"
    CASE 4
        IISStateText = "Stopped"
    CASE 5
        IISStateText = "Pausing"
    CASE 6
        IISStateText = "Paused"
    CASE 7
        IISStateText = "Continuing"
    CASE ELSE
        IISStateText = "Unknown state"
    END SELECT
End Function

Function FileSizeText(FileSizeBytes)
	If FileSizeBytes < 1024 Then
		FileSizeText = FileSizeBytes & " B"
	ElseIf FileSizeBytes < (1024 * 1024) Then
		FileSizeText = Round(FileSizeBytes / 1024.0, 3) & " KB"
	ElseIf FileSizeBytes < (1024 * 1024 * 1024) Then
		FileSizeBytes = FileSizeBytes / 1024
		FileSizeText = Round(FileSizeBytes / 1024.0, 3) & " MB"
	Else
		FileSizeBytes = FileSizeBytes / (1024 * 1024)
		FileSizeText = Round(FileSizeBytes / 1024.0, 3) & " GB"
   End If
End Function

Function OpenXHTMLDocument(Title)
	Response.Write "<?xml version=""1.0"" encoding=""iso-8859-1"" ?>"
	Response.Write "<!DOCTYPE html PUBLIC ""-//W3C//DTD XHTML 1.1//EN"" ""http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd"">"
	Response.Write "<html>"
	Response.Write "<head>"
	Response.Write "<title>" & Server.HTMLEncode(Title) & "</title>"
	Response.Write "</head>"
	Response.Write "<body>"
End Function

Function CloseXHTMLDocument()
	Response.Write "</body>"
	Response.Write "</html>"
End Function
%>
